import sys
import os
import fudge
import requests

sys.path.insert(0, os.path.abspath('..'))
from flyingsphinx import API, __version__

@fudge.patch('requests.get')
def test_get(get_method):
  response = fudge.Fake('Response')
  response.provides('json').returns({'qux': 'quux'})

  get_method.expects_call().with_args(
    'https://flying-sphinx.com/api/my/app/path', {'id': '55'}, {
      'Accept':                  'application/vnd.flying-sphinx-v3+json',
      'X-Flying-Sphinx-Token':   'abc:123',
      'X-Flying-Sphinx-Version': ('%s+python' % __version__)
    }
  ).returns(response)

  api = API('abc', '123')
  api.get('path', {'id': '55'})

@fudge.patch('requests.get')
def test_get_response(get_method):
  response = fudge.Fake('Response')
  response.provides('json').returns({'qux': 'quux'})
  get_method.is_callable().calls(lambda uri, params, headers: response)

  api = API('abc', '123')

  assert api.get('path') == {'qux': 'quux'}

@fudge.patch('requests.post')
def test_post(post_method):
  response = fudge.Fake('Response')
  response.provides('json').returns({'qux': 'quux'})

  post_method.expects_call().with_args(
    'https://flying-sphinx.com/api/my/app/start', {}, {
      'Accept':                  'application/vnd.flying-sphinx-v3+json',
      'X-Flying-Sphinx-Token':   'abc:123',
      'X-Flying-Sphinx-Version': ('%s+python' % __version__)
    }
  ).returns(response)

  api = API('abc', '123')
  api.post('start')

@fudge.patch('requests.post')
def test_post_response(post_method):
  response = fudge.Fake('Response')
  response.provides('json').returns({'qux': 'quux'})
  post_method.is_callable().calls(lambda uri, params, headers: response)

  api = API('abc', '123')

  assert api.post('path') == {'qux': 'quux'}

@fudge.patch('requests.put')
def test_put(put_method):
  response = fudge.Fake('Response')
  response.provides('json').returns({'qux': 'quux'})

  put_method.expects_call().with_args(
    'https://flying-sphinx.com/api/my/app/path', {'foo': 'bar'}, {
      'Accept':                  'application/vnd.flying-sphinx-v3+json',
      'X-Flying-Sphinx-Token':   'abc:123',
      'X-Flying-Sphinx-Version': ('%s+python' % __version__)
    }
  ).returns(response)

  api = API('abc', '123')
  api.put('path', {'foo': 'bar'})

@fudge.patch('requests.put')
def test_put_response(put_method):
  response = fudge.Fake('Response')
  response.provides('json').returns({'qux': 'quux'})
  put_method.is_callable().calls(lambda uri, params, headers: response)

  api = API('abc', '123')

  assert api.put('path') == {'qux': 'quux'}



import requests
import os
import re
import flyingsphinx

class API(object):
  URI_BASE   = 'https://flying-sphinx.com/api/my/app'
  API_ACCEPT = 'application/vnd.flying-sphinx-v3+json'

  def __init__(self, identifier = None, api_key = None):
    self.identifier = identifier or os.environ['FLYING_SPHINX_IDENTIFIER']
    self.api_key    = api_key    or os.environ['FLYING_SPHINX_API_KEY']

  def get(self, path, body={}):
    return self._send(requests.get, path, body)

  def post(self, path, body={}):
    return self._send(requests.post, path, body)

  def put(self, path, body={}):
    return self._send(requests.put, path, body)

  def _headers(self):
    return {
      'Accept':                  API.API_ACCEPT,
      'X-Flying-Sphinx-Token':   ('%s:%s' % (self.identifier, self.api_key)),
      'X-Flying-Sphinx-Version': ('%s+python' % flyingsphinx.__version__)
    }

  def _normalised_uri(self, path):
    path = '' if (path == '/') else ('/%s' % re.sub(r"^/", '', path))

    return API.URI_BASE + path

  def _send(self, http_method, path, body):
    return http_method(self._normalised_uri(path), body, self._headers()).json()



from os.path import basename

class Configuration(object):
  def __init__(self, api):
    self.api = api

  def upload(self, configuration):
    self.api.put('/', {'configuration': configuration})

  def upload_from_file(self, path):
    file = open(path)
    self.upload(file.read())

  def upload_settings(self, setting, name, contents):
    self.api.post('/add_file', {
      'setting':   setting,
      'file_name': name,
      'content':   contents
    })

  def upload_settings_from_file(self, setting, path):
    file = open(path)
    self.upload_settings(setting, basename(file.name), file.read())



import sys
import os
import fudge

sys.path.insert(0, os.path.abspath('..'))
from flyingsphinx import Index

@fudge.test
def test_index_start():
  api = fudge.Fake('API')
  api.provides('get').returns({'status': 'FINISHED', 'log': 'indexer output'})

  api.expects('post').with_args('indices', {'indices': ''})\
    .returns({'id': '55'})

  Index(api).run()

@fudge.test
def test_index_checks():
  api = fudge.Fake('API')
  api.provides('post').returns({'id': '55'})

  api.expects('get').with_args('indices/55').returns({
    'status': 'FINISHED',
    'log':    'indexer output'
  })

  Index(api).run()

@fudge.patch('time.sleep')
def test_index_checks_more_than_once(sleep):
  api = fudge.Fake('API')
  api.provides('post').returns({'id': '55'})
  api.provides('get').with_args('indices/55').returns({'status': 'PENDING'})\
    .next_call().returns({'status': 'FINISHED', 'log': 'indexer output'})

  sleep.expects_call().with_args(3)

  Index(api).run()

@fudge.patch('sys.stdout.write')
def test_index_prints_result(printer):
  api = fudge.Fake('API')
  api.provides('post').returns({'id': '55'})
  api.provides('get').with_args('indices/55').returns({
    'status': 'FINISHED',
    'log':    'indexer output'
  })

  printer.expects_call().with_args('indexer output\n')

  Index(api).run()

@fudge.patch('sys.stdout.write')
def test_index_prints_result(printer):
  api = fudge.Fake('API')
  api.provides('post').returns({'id': '55'})
  api.provides('get').with_args('indices/55').returns({'status': 'FAILED'})

  printer.expects_call().with_args('Index request failed.\n')

  Index(api).run()

@fudge.patch('sys.stdout.write')
def test_last_index_prints_result(printer):
  api = fudge.Fake('API')
  api.provides('get').with_args('indices/last').returns({
    'status': 'FINISHED',
    'log':    'indexer output'
  })

  printer.expects_call().with_args('indexer output\n')

  Index(api).last()


import sys
import os
import fudge

sys.path.insert(0, os.path.abspath('..'))
from flyingsphinx import Configuration

@fudge.test
def test_configuration_upload():
  api = fudge.Fake('API')
  api.expects('put').with_args('/', {'configuration': 'provided content'})
  Configuration(api).upload('provided content')

@fudge.patch('__builtin__.open')
def test_configuration_upload_from_file(fake_open):
  api       = fudge.Fake('API')
  conf_file = fudge.Fake('File')
  fake_open.is_callable().calls(lambda path: conf_file)
  conf_file.provides('read').returns('content in file')

  api.expects('put').with_args('/', {'configuration': 'content in file'})

  Configuration(api).upload_from_file('/path/to/file')

@fudge.test
def test_settings_upload():
  api = fudge.Fake('API')

  api.expects('post').with_args('/add_file',
    {'setting': 'wordforms', 'file_name': 'foo.txt', 'content': 'contents'})

  Configuration(api).upload_settings('wordforms', 'foo.txt', 'contents')

@fudge.patch('__builtin__.open')
def test_settings_upload_from_file(fake_open):
  api       = fudge.Fake('API')
  conf_file = fudge.Fake('File')
  fake_open.is_callable().calls(lambda path: conf_file)
  conf_file.provides('read').returns('contents')
  conf_file.name = '/path/to/foo.txt'

  api.expects('post').with_args('/add_file',
    {'setting': 'wordforms', 'file_name': 'foo.txt', 'content': 'contents'})

  Configuration(api).upload_settings_from_file('wordforms', '/path/to/foo.txt')



from sys import stdout
import time

class Index(object):
  def __init__(self, api):
    self.api = api

  def run(self, indices = []):
    id = self.api.post('indices', {'indices': ','.join(indices)})['id']

    result = self.api.get(('indices/%s' % id))
    while result['status'] == 'PENDING':
      time.sleep(3)
      result = self.api.get(('indices/%s' % id))

    if result['status'] == 'FAILED':
      stdout.write('Index request failed.\n')
    else:
      stdout.write('%s\n' % result['log'])

  def last(self):
    result = self.api.get('indices/last')
    stdout.write('%s\n' % result['log'])



import sys
import os
import fudge

sys.path.insert(0, os.path.abspath('..'))
from flyingsphinx import Sphinx

@fudge.test
def test_start():
  api = fudge.Fake('API')
  api.expects('post').with_args('start')
  Sphinx(api).start()

@fudge.test
def test_stop():
  api = fudge.Fake('API')
  api.expects('post').with_args('stop')
  Sphinx(api).stop()



import sys
import os
import fudge

sys.path.insert(0, os.path.abspath('..'))
import flyingsphinx

@fudge.patch('flyingsphinx.API')
def test_info_api_call(api_class):
  api = fudge.Fake('API')
  api_class.is_callable().calls(lambda: api)

  api.expects('get').with_args('/')

  flyingsphinx.info()

@fudge.patch('flyingsphinx.API')
def test_info_result(api_class):
  api = fudge.Fake('API')
  api_class.is_callable().calls(lambda: api)
  api.provides('get').with_args('/').returns({'bar': 'baz'})

  assert flyingsphinx.info() == {'bar': 'baz'}



#!/usr/bin/env python

import os
import sys

from setuptools import setup

os.environ['PYTHONDONTWRITEBYTECODE'] = '1'

setup(
  name                 = 'flyingsphinx',
  version              = '0.0.10',
  description          = 'Flying Sphinx Python client',
  long_description     = 'Flying Sphinx API client for Python applications',
  author               = 'Pat Allan',
  author_email         = 'pat@freelancing-gods.com',
  url                  = 'https://github.com/flying-sphinx/flying-sphinx-py',
  packages             = ['flyingsphinx'],
  # package_data         = {'': ['LICENCE', 'README.textile']},
  include_package_data = True,
  install_requires     = ['requests'],
  license              = open('LICENCE').read(),
  classifiers          = (
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',
    'Natural Language :: English',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2.7'
  )
)



"""
flyingsphinx
~~~~~~~~~~~~

:copyright: (c) 2012 by Pat Allan
:license: MIT, see LICENCE for more details.
"""

__title__     = 'flyingsphinx'
__version__   = '0.0.10'
__author__    = 'Pat Allan'
__license__   = 'MIT'
__copyright__ = 'Copyright 2012 Pat Allan'

from .api           import API
from .configuration import Configuration
from .index         import Index
from .sphinx        import Sphinx

def configuration():
  return Configuration(API())

def index():
  return Index(API())

def info():
  return API().get('/')

def sphinx():
  return Sphinx(API())
